@@ -6,7 +6,7 @@ import numpy as np
 from flax.linen import linear as flax_module
 
 from treex import types
-from treex.module import Module
+from treex.module import Module, next_key
 
 
 class Linear(Module):
@@ -21,11 +21,10 @@ class Linear(Module):
     """
 
     # pytree
-    kernel: types.Parameter[jnp.ndarray, None]
-    bias: types.Parameter[jnp.ndarray, None]
+    kernel: tp.Optional[jnp.ndarray] = types.Parameter.node()
+    bias: tp.Optional[jnp.ndarray] = types.Parameter.node()
 
     # static
-    features_in: int
     features_out: int
     use_bias: bool
     dtype: tp.Any
@@ -41,8 +40,8 @@ class Linear(Module):
 
     def __init__(
         self,
-        features_in: int,
         features_out: int,
+        *,
         use_bias: bool = True,
         dtype: tp.Any = jnp.float32,
         precision: tp.Any = None,
@@ -54,6 +53,7 @@ class Linear(Module):
             [flax_module.PRNGKey, flax_module.Shape, flax_module.Dtype],
             flax_module.Array,
         ] = flax_module.zeros,
+        name: tp.Optional[str] = None,
     ):
         """
         Arguments:
@@ -66,9 +66,7 @@ class Linear(Module):
             kernel_init: initializer function for the weight matrix.
             bias_init: initializer function for the bias.
         """
-        super().__init__()
-
-        self.features_in = features_in
+        super().__init__(name=name)
         self.features_out = features_out
         self.use_bias = use_bias
         self.dtype = dtype
@@ -90,21 +88,7 @@ class Linear(Module):
             bias_init=self.bias_init,
         )
 
-    def module_init(self, key: jnp.ndarray):
-        batch_size = 10  # random
-        x = jax.random.uniform(key, (batch_size, self.features_in))
-
-        variables = self.module.init(key, x)
-
-        # Extract collections
-        params = variables["params"].unfreeze()
-
-        self.kernel = params["kernel"]
-
-        if self.use_bias:
-            self.bias = params["bias"]
-
-    def __call__(self, x: np.ndarray) -> jnp.ndarray:
+    def __call__(self, x: jnp.ndarray) -> jnp.ndarray:
         """Applies a linear transformation to the inputs along the last dimension.
 
         Arguments:
@@ -113,7 +97,16 @@ class Linear(Module):
         Returns:
             The transformed input.
         """
-        assert self.initialized, "Module is not initialized."
+        if self.initializing():
+            variables = self.module.init({"params": next_key()}, x)
+
+            # Extract collections
+            params = variables["params"].unfreeze()
+
+            self.kernel = params["kernel"]
+
+            if self.use_bias:
+                self.bias = params["bias"]
 
         assert self.kernel is not None
         params = {"kernel": self.kernel}
